@@ -8,7 +8,8 @@ from mpl_toolkits.mplot3d.art3d import Poly3DCollection
 
 from skimage import measure
 from skimage.draw import ellipsoid
-
+from scipy import ndimage 
+import pyvista as pv
 
 def findcrossover(array,low,high,value):
     'finds crossover index of array for `value` value'
@@ -69,6 +70,85 @@ def tensor2crds(tensor,mult):
     return np.array([ [*i*mult] for i in np.argwhere(tensor)])
 
 
+def resize_array(array, mult=(2,2,2)):
+    '''Resizes a three-dimensional array by the three dim factors specified by `mult` tuple. 
+    Converts to sparse array of 0s and 1s   
+
+    Parameters
+    ----------
+    array : np.array(int,int,int)
+        array to resize
+    mult: tuple(float,float,float)
+        depth length width factors to resize array with. e.g 2,2,2 resizes array to double its size in all dims
+    '''
+    
+    unique = np.unique(array)
+
+    array = ndimage.zoom(array, mult).astype('int')
+    crds_nonzero = np.argwhere(array > 0)
+    src = array.copy()
+
+    print(np.unique(src))
+    array.fill(0)
+
+    for k in crds_nonzero:
+        if src[tuple(k)] in unique:
+            array[tuple(k)] = src[tuple(k)]
+
+    return array
+
+
+def random_kernel_convolve(array, kernel,random_bounds=(-10,10)):
+    '''Applies a three-dimensional convolution with a randomly-mutating `kernel` 
+    on a 3-D `array` which changes for every array site when random_bounds are set to tuple. 
+    If random_bounds are set to False, convolution occurs in constant mode for the specified kernel. 
+
+    Parameters
+    ----------
+    array : np.array(int,int,int)
+        array to convolve
+    kernel: np.array(int,int,int)
+        kernel to use for convolution. If random_bounds are set to tuple, only the kernel's shape is used to specify the random_kernels
+    random_bounds : tuple(int,int) OR bool
+        see above explanation.
+    '''
+    if random_bounds and kernel.shape[0]%2:	
+        new_array = np.zeros(array.shape)
+
+        array = np.pad(array,kernel.shape[0]//2)
+
+        k_z,k_y,k_x = kernel.shape
+        Z,Y,X = new_array.shape 
+        # adapted from: 
+        # https://towardsdatascience.com/tensorflow-for-computer-vision-how-to-implement-convolutions-from-scratch-in-python-609158c24f82
+        for k in range(Z):
+            for j in range(Y):
+                for i in range(X):
+                    sector = array[k:k+k_z, j:j+k_y, i:i+k_x]
+
+                    kernel = np.random.randint(*random_bounds,(k_z,k_y,k_x)) 
+                    new_array[k,j,i] = np.sum(np.multiply(sector, kernel))
+        return new_array
+
+    else:
+        return ndimage.convolve(array, kernel, mode='constant', cval=0.0)
+
+def roughen(array,kernel_level=1):
+    '''Makes a 3d array model rougher by a special convolution operation. Uses `voxelmap.random_kernel_convolve`.
+
+    Parameters
+    ----------
+    array : np.array(int,int,int)
+        array to `roughen up`
+    kernel_level: int
+        length scale (size) of random kernels used. The smallest scale (=1) gives the roughest transformation.
+    '''
+    kernel = np.zeros((kernel_level,kernel_level,kernel_level))
+    return random_kernel_convolve(array,kernel,(-1,2))
+
+
+
+
 def load_array(filename):
     '''Loads a pickled numpy array with `filename` name'''
     return pickle.load( open(filename, "rb" ),encoding='latin1')
@@ -269,39 +349,44 @@ def MarchingMesh(
     # Display resulting triangular mesh using Matplotlib. This can also be done
     # with mayavi (see skimage.measure.marching_cubes_lewiner docstring).
     if plot:
-        fig = plt.figure(figsize=figsize)
-        ax = fig.add_subplot(111, projection='3d')
+        # fig = plt.figure(figsize=figsize)
+        # ax = fig.add_subplot(111, projection='3d')
 
-        # Fancy indexing: `verts[faces]` to generate a collection of triangles
-        print(verts[faces-1])
-        mesh = Poly3DCollection(verts[faces-1])
-        mesh.set_edgecolor('k')
-        ax.add_collection3d(mesh)
+        # # Fancy indexing: `verts[faces]` to generate a collection of triangles
+        # print(verts[faces-1])
+        # mesh = Poly3DCollection(verts[faces-1])
+        # mesh.set_edgecolor('k')
+        # ax.add_collection3d(mesh)
 
-        def maxmin(arr): return np.min(arr), np.max(arr)
+        # def maxmin(arr): return np.min(arr), np.max(arr)
 
-        ax.set_xlim(*maxmin(verts.T[0]))  
-        ax.set_ylim(*maxmin(verts.T[1])) 
-        ax.set_zlim(*maxmin(verts.T[2])) 
+        # ax.set_xlim(*maxmin(verts.T[0]))  
+        # ax.set_ylim(*maxmin(verts.T[1])) 
+        # ax.set_zlim(*maxmin(verts.T[2])) 
 
-        plt.tight_layout()
-        plt.show()
+        # plt.tight_layout()
+        # plt.show()
+
+        MeshView(out_file)
 
 
-
-import voxelmap.objviewer as viewer
-
-def MeshView(objfile='model.obj', wireframe=False, viewport=(2048, 1152)):
+def MeshView(objfile='model.obj',wireframe=False,color='pink',alpha=0.5,background_color='#333333', viewport = [1024, 768]):
     '''[GLOBAL_METHOD] 
-    MeshView: triangulated mesh view with OpenGL [ uses pygame ]
-
+    MeshView: triangulated mesh view with PyVista 
     Parameters
     ----------
     objfile: string
         .obj file to process with MeshView [in GLOBAL function only]
     wireframe: bool
         Represent mesh as wireframe instead of solid polyhedron if True (default: False). 
+    color : string / hexadecimal
+        mesh color. default: 'pink'
+    alpha : float
+        opacity transparency range: 0 - 1.0. Default: 0.5
+    background_color : string / hexadecimal
+        color of background. default: 'pink'
     viewport : (int,int)
         viewport / screen (width, height) for display window (default: 80% your screen's width & height)
     '''
-    viewer.objview(objfile, wireframe=wireframe, usemtl=False, viewport=viewport)
\ No newline at end of file
+    mesh = pv.read(objfile)
+    mesh.plot(show_edges=True if wireframe else False, color=color,opacity=alpha,background=background_color,window_size = viewport)
\ No newline at end of file
